import java.io.BufferedReader;
import java.io.InputStreamReader;

import org.apache.log4j.Logger;

import proxy.ListeningThread;
import util.Global;
import util.ParseConfiguration;

import static util.Constant.DEFAULT_LISTENING_PORT;
import static util.Constant.STR_VERSION;
import static util.Constant.CONFIGURATION_FILE;

/**
 * A small fake HTTP proxy server designed to be used whith receivers that support
 * Net-Tune(tm) protocol.
 * 
 * I found this piece of code on internet, it was writen by Julien Chable. Originaly,
 * it was a simple HTTP proxy fowarder.
 *  
 * @author CHABLE Julien, adaptation for net-tune by Olivier Garrouste.
 * @version 1.1
 */

public class StartServer {

	private static final Logger logger = Logger.getLogger(StartServer.class);
	
	/**
	 * Program entry point.
	 * 
	 * @param args Command line arguments.
	 */
	public static void main(String args[]) {
		ListeningThread proxyServer;
		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
		String command;
		boolean ok;

		System.out.println("ProxyServer for Net-Tune protocol. Program version " + STR_VERSION + ".");

		// Retrieve command line options.
		ok = parseCommandLine(args);
		if ( !ok ) {
			logger.debug("End.");
			return;
		}

		// Read the configuration file.
		ParseConfiguration.parse(CONFIGURATION_FILE);

		// Create the listening server.
		proxyServer = new ListeningThread(Global.listeningPort);

		// Display the console commands.
		displayConsoleCommands(Global.listeningPort);

		// Now the console: wait for the user to press exit.
		try {
			while (true) {
				System.out.print(">");
				command = input.readLine();

				if ("exit".equalsIgnoreCase(command))
					break;
				else if ( "t".equalsIgnoreCase(command) )
					displayThreads();
				else
					displayConsoleCommands(Global.listeningPort);
			}
		}
		catch (Exception e) {
			logger.error("Error : " + e.toString());
		}

		proxyServer.stopThread();
		logger.debug("End.");
	}
	
	/**
	 * Parse command line.
	 * 
	 * @param args The comand line arguments.
	 * @return True if continue running appplication, false exit application.
	 */
	private static boolean parseCommandLine(String args[]) {
		int i, nb;
		
		nb = args.length;

		// Test each argument.
		i = 0;
		while ( i < nb ) {
			if ( "-h".equals(args[i]) ) {
				usage();
				return false;
			}

			if ( "-port".equals(args[i]) ) {
				// Next argument.
				i++;
				
				try {
					// Convert the port to an int.
					Global.listeningPort = Integer.parseInt(args[i]);
				}
				catch (Exception e) {
					System.out.println("Invalid port number (" + args[i] +
						"), using default port " + DEFAULT_LISTENING_PORT + ".");
					Global.listeningPort = DEFAULT_LISTENING_PORT;
					
					// Does not take this parameter into account.
					continue;
				}
			}
			else if ( "-nometadata".equals(args[i]) )
				Global.allowMp3Metadata = false;
			else if ( "-disable".equals(args[i]) )
				Global.enableFiltering = false;

			// Next command line option.
			i++;
		}
		return true;
	}

	/**
	 * Display the help.
	 */
	private static void usage() {
		System.out.print(
			"Usage: java -jar NetTuneProxy.jar [-disable] [-h] [-port x] [-nometadata]\n" +
			"   -disable      disable interception of requests to host \"www.xiva-net.com\". Forward\n" +
			"                 all requests whithout modification. Default is enabled.\n" +
			"   -h            this help.\n" +
			"   -nometadata   filter out metadata in mp3 streams. If you hear regular glitches when\n" +
			"                 listening to a mp3 stream, try enabling this option. It has no effect\n" +
			"                 if -disable is set.\n" +
			"   -port         listen on port x, default is " + DEFAULT_LISTENING_PORT + ".\n");
	}

	/**
	 * Display the concole commands.
	 * 
	 * @param listeningPort
	 */
	private static void displayConsoleCommands(int listeningPort) {
		System.out.print(
			"Server listening on port " + listeningPort + ".\n" +
			"Press \"exit\" to stop the proxy server.\n" +
			"Press \"t\" to list running threads.\n");
	}

	public static void displayThreads() {
	    ThreadGroup threadGroup;
	    ThreadGroup parent = null;
	    Thread threads[];
	    int i;

	    // Get the current thread group.
	    threadGroup = Thread.currentThread().getThreadGroup();

	    while ( true ) {
	    	parent = threadGroup.getParent();
	    	
	    	if ( parent == null )
	    		break;

	    	threadGroup = parent;
	    }

	    threads = new Thread[threadGroup.activeCount()];

	    // Store all the threads in the array.
	    threadGroup.enumerate(threads);

	    System.out.println("Running threads:");
	    for ( i=0 ; i<threads.length ; i++ ) {
	      if ( threads[i] != null )
        	System.out.println(" - " + threads[i].getName());
	    }
	  }

}