package dao;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Logger;

import valueObject.AbstractGenreValueObject;
import valueObject.GenreValueObject;


/**
 * Data manipulation for GenreValueObject.
 * Storage place of GenreValueObjects (memory database).
 * 
 * @author Olivier Garrouste.
 * Creation date: 2006-02-25.
 */
public class GenreDao {

	private static final Logger logger = Logger.getLogger(GenreDao.class);

	// Sequence.
	private static int code;

	// Genre map <code,GenreValueObject>.
	private static HashMap<String,AbstractGenreValueObject> hashmapCode;
	
	// Genre map <name,GenreValueObject>.
	private static HashMap<String,AbstractGenreValueObject> hashmapName;
	
	// Database initialization.
	static {
		clear();
	}

	private static class GenreByNameComparator implements Comparator<AbstractGenreValueObject> {
		public int compare(AbstractGenreValueObject g1, AbstractGenreValueObject g2) {
			return g1.getName().compareTo(g2.getName());
		}
	}

	/**
	 * Clear the sequence and storage of genre objects.
	 */
	public static synchronized void clear() {
		code = 1;
		hashmapCode = new HashMap<String,AbstractGenreValueObject>();
		hashmapName = new HashMap<String,AbstractGenreValueObject>();
	}

	/**
	 * Return the next unique genre identifier.
	 * 
	 * @return The next unique genre identifier
	 */
	public static synchronized int getNextCode() {
		return code++;
	}

	/**
	 * Add a genre into the Genre database, replace any existing one.
	 * 
	 * @param genre A GenreValueObject.
	 */
	public static synchronized void insert(GenreValueObject genre) {

		// Nothing to store, exit.
		if ( genre == null )
			return;

		// Store.
		hashmapCode.put(genre.getCode(), genre);
		hashmapName.put(genre.getName(), genre);
	}

	/**
	 * Return the GenreValueObject of the given name. If it does not exist return null.
	 * 
	 * @param name The name of the genre requested.
	 * @return A GenreValueObject or null if it does not exists.
	 */
	public static AbstractGenreValueObject getByName(String name) {
		return hashmapName.get(name);
	}

	/**
	 * Return the list of the genres ordered by code.
	 * 
	 * @return An ArrayList of GenreValueObject.
	 */
	/*public static ArrayList<AbstractGenreValueObject> getListOrderByCode() {
		ArrayList<AbstractGenreValueObject> list;
		
		list = new ArrayList<AbstractGenreValueObject>();
		list.addAll(hashmapCode.values());

		return list;
	}*/

	/**
	 * Return the list of the genres ordered by name.
	 * 
	 * @return An ArrayList of GenreValueObject.
	 */
	public static AbstractGenreValueObject[] getListOrderByName() {
		AbstractGenreValueObject abstractGenre[];

		// Convert hashmap to array.
		abstractGenre = hashmapCode.values().toArray(new AbstractGenreValueObject[0]);

		// Sort array by genre name.
		Arrays.sort(abstractGenre, new GenreByNameComparator());

		return abstractGenre;
	}

	/**
	 * Return the genre of the given code.
	 * 
	 * @param code
	 * @return A AbstractGenreValueObject or null if nothing corresponds to that code.
	 */
	public static AbstractGenreValueObject getByCode(String strCode) {
		
		return hashmapCode.get(strCode);
	}

	public static void dumpGenreByCode() {
		AbstractGenreValueObject genre;
		Set<String> keySet;
		Iterator<String> iterator;
		String strCode;

		keySet = hashmapCode.keySet();
		iterator = keySet.iterator();

		while ( iterator.hasNext() ) {
			strCode = iterator.next();
			genre = hashmapCode.get(strCode);

			logger.debug(genre.toString());
		}
	}
}
