package proxy;

import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import org.apache.log4j.Logger;

/**
 * Title: Proxy server
 * Description: Main classe of the proxy server. This class waits for incoming
 * client connections forever.
 * 
 * @author CHABLE Julien, Olivier Garrouste.
 * @version 1.0
 */
public class ListeningThread extends Thread {

	// A logger.
	private static final Logger logger = Logger.getLogger(ListeningThread.class);

	// The current listening socket.
	private ServerSocket serverSocket;

	// Listening port.
	private int port;


	/**
	 * Constructor.
	 * 
	 * @param port Port on which the proxy server will listen.
	 */
	public ListeningThread(int port) {
		// Name the thread.
		super("ListeningThread");

		this.port = port;
		
		// Start the thread.
		start();
	}

	/**
	 * Main loop : wait incoming connections forever or until stopThread() is called.
	 */
	public void run() {
		Socket clientSocket;

		logger.debug("Proxy server started.");
		try {
			serverSocket = new ServerSocket(port);

			// Wait for new incoming client connections.
			while ( !interrupted() ) {
				clientSocket = serverSocket.accept();
				logger.info("New incoming connection from " + clientSocket.toString());

				// Process the requests on this connection in a separate thread.
				new ProxyServerClientThread(clientSocket);

				// Wait 5 milliseconds before accepting another connection.
				sleep(5);
			}
		}
		catch (BindException e) {
			logger.fatal(e.toString() + ". The listening port is probably already in " +
				"use. Stopping proxy server.");
			System.exit(1);
		}
		catch (SocketException e) {
			// Do nothing, hide the exception thown when server is stopped.
		}
		catch (Exception e) {
			logger.error("ERROR : " + e.toString(), e);
		}
		logger.debug("Listening thread stoped.");
	}

	/**
	 * Stop the server thread.
	 */
	public void stopThread() {
		try {
			// Closes the listening socket. If the thread was waiting in the accept()
			// method a SocketException is thrown in the run method.
			serverSocket.close();
		}
		catch (IOException e) {
			logger.error("ERROR : " + e.toString(), e);
		}
	}
}