package proxy;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.URL;

import org.apache.log4j.Logger;

import proxy.streamHandler.DefaultStreamHandler;
import proxy.streamHandler.IcyStreamHandler;
import proxy.streamHandler.NetTuneStreamHandler;
import util.Constant;
import util.Global;
import util.Net;
import util.ProtocolAnalyzer;
import util.RequestParser;
import util.Constant.Protocol;
import valueObject.ResponseValueObject;
import valueObject.ProxyRequestValueObject;

/**
 * Title: Proxy thread
 * Description: a proxy is a relay between the net-tune client and the server.
 * Filters client requests :
 * - if they are addressed to Xiva, the proxy filters them and respond according to the
 *   configuration file.
 * - if they are not addressed to Xiva, the proxy forward them.
 * 
 * If the requested url is a mp3 stream, the proxy may filters the returned server data
 * before sending it to the client (see the documentation for more information).
 * 
 * Filtering can be disabled (see Global.enableFiltering).
 * 
 * @author CHABLE Julien, Olivier Garrouste
 * @version 1.1
 */
public class ProxyServerClientThread extends Thread {

	private static final Logger logger = Logger.getLogger(ProxyServerClientThread.class);
	
	private Socket clientSocket, socketServer;
	private InputStream clientIn;
	private OutputStream clientOut;

	// Unique id of the next thread.
	private static int threadId = 0;

	// Number of client thread currently running. Expected operations on this variable
	// to be atomic as I don't want to deal with thread synchronization.
	private static int runningThreads = 0;


	/**
	 * Constructor.
	 * 
	 * @param socket The incoming net-tune client socket.
	 */
	public ProxyServerClientThread(Socket clientSocket) {
		// Name the thread.
		super("ProxyServerClientThread-" + (threadId++));

		this.clientSocket = clientSocket;

		try {
			// Get the streams assciated with this client connection.
			clientIn = clientSocket.getInputStream();
			clientOut = clientSocket.getOutputStream();
		}
		catch (IOException e) {
			logger.error("Erreur IO Client Proxy Server : " + e.toString());
		}

		// Start the thread.
		start();
	}

	/**
	 * Thread entry point.
	 */
	public void run() {
		// Increase running thread count.
		runningThreads++;

		// Call the main processing method.
		process();

		// Close all remaining connections.
		closeAll();
		runningThreads--;
	}

	/**
	 * Main processing.
	 */
	private void process() {
		ProxyRequestValueObject proxyRequest;
		boolean ok;

		// Obtain and transform the proxy-style client request to standard HTTP request.
		proxyRequest = RequestParser.build(clientIn, clientOut);

		// Check current number of client threads.
		if ( runningThreads > Constant.MAX_CLIENT_THREAD ) {
			logger.warn("Too many incoming client connections(" + runningThreads +
				"), increase MAX_CLIENT_THREAD.");

			// There is too many threads running, send an error to the client.
			Net.sendHttp(clientOut, "NetTuneProxy: connection refused, too many clients.");
			return;
		}

		// Client request is invalid...
		if ( proxyRequest == null ) {
			// Send back an error to the client.
			Net.sendHttp(clientOut, "NetTuneProxy: invalid request.\n");
			return;
		}


		//
		// Filters requests to www.xiva-net.com.
		//

		// Filtering is turned on.
		if ( Global.enableFiltering ) {
			// Is it a request to the destinated to Xiva ?
			if ( isXivaRequest(proxyRequest) ) {
				logger.info("Intercepted Xiva request : " + proxyRequest.getServerUrl().toString());

				// Process the request.
				NetTuneStreamHandler.process(proxyRequest, clientIn, clientOut);
				return;
			}

			// Request not filtered, forward it.
			handleRequest(proxyRequest);
			return;
		}

		// Filtering not enabled.

		// Connect to server.
		ok = connectToServer(proxyRequest);
		
		// Could not connect to server, exit.
		if ( !ok )
			return;

		// Send the client request to the server.
		new ProxyToServerThread(proxyRequest);

		// Use default stream handler (direct forwarding from server to client).
		DefaultStreamHandler.process(proxyRequest, null);
	}

	/**
	 * Handle requests not destinated to Xiva.
	 *
	 * @param proxyRequest
	 */
	private void handleRequest(ProxyRequestValueObject proxyRequest) {
		ResponseValueObject response;
		boolean ok;

		// Connect to server.
		ok = connectToServer(proxyRequest);
		
		// Could not connect to server, exit.
		if ( !ok )
			return;

		try {
			// Send the client request to the server.
			new ProxyToServerThread(proxyRequest);

			// Analyze protocol in server response.
			response = ProtocolAnalyzer.analyze(proxyRequest);

			// If shoutcast protocol is used, use particular stream handler.
			if ( response.getIdProtocol() == Protocol.ICY ) {
				logger.debug("icy-metaint=" + response.getHeader("icy-metaint") + ".");
				// Test if an error has been returned. Some Shoutcast servers respond
				// with error code 404 when the request did not contains the
				// "icy-metaData" header with a value of 1.
				if ( response.getStatusCode() == 404 ) {
					logger.debug("Error 404 encountered, forcing icy-metadata header in request.");
					// Add the header.
					proxyRequest.appendRequestLine("Icy-Metadata:1");

					// Send the modified client request to the server.
					new ProxyToServerThread(proxyRequest);

					// Analyze protocol in server response.
					response = ProtocolAnalyzer.analyze(proxyRequest);

					// No error, it is a regular stream, transmit it.
					if ( response.getIdProtocol() == Protocol.ICY ) {
						logger.debug("Process ICY stream.");
						IcyStreamHandler.process(proxyRequest, response);
					}
				}
				else
					IcyStreamHandler.process(proxyRequest, response);
			}
			else
				// Default stream handler (direct forwarding).
				DefaultStreamHandler.process(proxyRequest, response.getData());
		}
		catch (Exception e) {
			logger.error(e.toString(), e);
		}
	}

	/**
	 * Connect to the server designated in the ProxyRequestValueObject.
	 * 
	 * @param proxyRequest A valid ProxyRequestValueObject.
	 * @return true if the connection to the server was successful, fills the fiels
	 * serverInputStream and serverOutputStream. If the connection could not be
	 * established, send back an error to the client and return false.
	 */
	private boolean connectToServer(ProxyRequestValueObject proxyRequest) {
		OutputStream serverOut;
		InputStream serverIn;
		int serverPort;

		
		// If no server port is specified, use default port 80.
		serverPort = proxyRequest.getServerUrl().getPort();

		if ( serverPort <= 0 )
			serverPort = 80;

		try {
			// Connect to the server.
			socketServer = new Socket(proxyRequest.getServerUrl().getHost(), serverPort);

			// Get the input and output streams.
			serverIn = socketServer.getInputStream();
			serverOut = socketServer.getOutputStream();

			// Set them in the proxyRequest object.
			proxyRequest.setServerInputStream(serverIn);
			proxyRequest.setServerOutputStream(serverOut);
		}
		catch (Exception e) {
			// Socket exceptions are 'almost' normal operations, I log them in info mode.
			logger.info(e.toString(), e);

			// Send back an error to the client.
			Net.sendHttp(proxyRequest.getClientOutputStream(), "Request=" +
				proxyRequest.getRequest() + "<br>\n<br>\nerror=" + e.toString());

			// Close all remaining connections and exit.
			closeAll();
			return false;
		}

		return true;
	}

	/**
	 * If the request is destinated to Xiva (http://www.xiva-net.com/services/irds.jsp),
	 * process it and return true. Else return false.
	 * 
	 * @param proxyRequest A ProxyRequestValueObject holding the net-tune client request.
	 * @return If the request is destinated to Xiva, process it and return true. Else
	 * return false.
	 */
	private static boolean isXivaRequest(ProxyRequestValueObject proxyRequest) {
		URL serverUrl;

		// Return the server URL (only if the method was HTTP GET), else return
		// null.
		serverUrl = proxyRequest.getServerUrl();

		// It is not a GET command, we don't process it.
		if ( serverUrl == null )
			return false;
		
		// Filters requests to xiva-net.com.
		if ( serverUrl.getHost().indexOf("www.xiva-net.com") == -1 )
			return false;

		// Accept only the page /services/irds.jsp, other pages are sent to xiva.
		if ( !"/services/irds.jsp".equals(serverUrl.getPath()) )
			return false;

		return true;
	}
	
	/**
	 * Close all connections.
	 */
	private void closeAll() {
		if ( clientIn != null ) {
			try {
				clientIn.close();
				clientIn = null;
			}
			catch (Exception e) {
				clientIn = null;
				logger.error("Error closing clientIn.");
			}
		}

		if ( clientOut != null ) {
			try {
				clientOut.close();
				clientOut = null;
			}
			catch (Exception e) {
				clientOut = null;
				logger.error("Error closing clientOut.");
			}
		}

		if ( clientSocket != null ) {
			try {
				clientSocket.close();
				clientSocket = null;
			}
			catch (Exception e) {
				clientSocket = null;
				logger.error("Error closing clientSocket.");
			}
		}

		if ( socketServer != null ) {
			try {
				socketServer.close();
				socketServer = null;
			}
			catch (Exception e) {
				socketServer = null;
				logger.error("Error closing socketServer.");
			}
		}
	}
}