package proxy.streamHandler;

import java.io.InputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;

import util.Global;
import util.Net;
import valueObject.ResponseValueObject;
import valueObject.ProxyRequestValueObject;

/**
 * Title: Proxy server
 * Description: Process data exchange between client and server for the ICY (shoutcast)
 * protocol.
 * 
 * @author Olivier Garrouste
 * Creation date: 2006-03-26
 */
public class IcyStreamHandler {

	private static final Logger logger = Logger.getLogger(IcyStreamHandler.class);

	/**
	 * ICY protocol stream handler: read data from server and send it to the net-tune
	 * client. Optionally filters metadata if the option is set.
	 * 
	 * @param proxyRequest A valid ProxyRequestValueObject object.
	 * @param protocol A valid ResponseValueObject object.
	 */
	public static void process(ProxyRequestValueObject proxyRequest, ResponseValueObject protocol) {
		OutputStream clientOut;
		InputStream serverIn;
		String str;
		int metaInterval, nextMetadata;

		serverIn = proxyRequest.getServerInputStream();
		clientOut = proxyRequest.getClientOutputStream();

		// The user set the option he wants metadata.
		if ( Global.allowMp3Metadata ) {
			metaInterval = -1;
			nextMetadata = 0;
		}
		else {
			// Get the meta-data interval.
			str = protocol.getHeader("icy-metaint");
	
			// No icy-metaint header: there is no meta-data in this stream.
			if ( str == null ) {
				metaInterval = -1;
				nextMetadata = 0;
			}
			else {
				try {
					metaInterval = Integer.parseInt(str);

					// Place of the next meta-data in the stream (because there were some
					// data read with the headers).
					nextMetadata = metaInterval - protocol.getData().length;
				}
				catch (Exception e) {
					logger.warn("Meta-data interval invalid: \"" + str + "\".");
					metaInterval = -1;
					nextMetadata = 0;
				}
			}
		}

		logger.debug("Start reading from server and sending to client, " +
			"allowMp3Metadata=" + Global.allowMp3Metadata + ".");

		// Some data was read to analyze protocol, we must send it first.
		Net.write(protocol.getData(), clientOut);

		// Read response from server and forward to client.
		transfert(serverIn, clientOut, metaInterval, nextMetadata);
		logger.debug("End reading from server and sending to client.");
	}

	/**
	 * Transfert from inputStream to outputStream as long as there is data to transfert
	 * and possibly removing meta-data. The meta-data consists of one byte indicating
	 * the size of the meta-data frame divided by 16. We must not send these data to
	 * the client.
	 * 
	 * @param inputStream The source stream to read from.
	 * @param outputStream The destination stream to write to.
	 * @param metadataInterval Interval between the beginning of each metadata in the
	 * source stream.
	 * @param nextMetadata The place of the next meta-data the stream, -1 means no
	 * metadata removing (direct copy from input stream to output stream).
	 */
	private static void transfert(InputStream inputStream, OutputStream outputStream,
			int metadataInterval, int nextMetadata) {
		int metadataSize;
		boolean ok;

		ok = true;

		try {
			// No meta-data to remove from stream.
			if ( metadataInterval == -1 ) {
				// Read response from server and forward to client forever.
				Net.transfert(inputStream, outputStream);
				return;
			}

			// Meta-data removing requested.
			while (ok) {
				// Transfert the music data stopping at the begining of meta-data.
				ok = Net.transfert(inputStream, outputStream, nextMetadata);
				
				// End of stream or error, exit from loop.
				if ( !ok )
					break;

				metadataSize = inputStream.read();
				logger.debug("meta-data byte=" + metadataSize + ".");

				if ( metadataSize != 0 ) {
					metadataSize *= 16;
	
					// Read the meta-data but does nothing with it.
					Net.read(inputStream, metadataSize);
					//logger.debug("metadata content=" + new String(buffer) + ".");
				}

				nextMetadata = metadataInterval;
			}
		}
		catch (Exception e) {
			logger.error("ERROR : " + e.toString(), e);
		}
	}


}
