package proxy.streamHandler;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import org.apache.log4j.Logger;

import util.Constant;
import util.Net;
import util.NetTuneProtocol;
import util.ParseConfiguration;
import util.QueryString;
import valueObject.ProxyRequestValueObject;

/**
 * Title: Proxy server
 * Description: Process data exchange between a Net-Tune client and server. The request
 * is not send to the server but processed localy.
 * 
 * On the first access to iRadio feature, the Net-Tune receiver sends the following
 * request: /~prullman/irds.jsp
 * - We return the list of Genres, Locations, Languages and Formats.
 * 
 * If you select Genre on the receiver, the receiver sends the following request:
 * /~prullman/irds.jsp?Format=xxx&Genre=yyy
 * - We return the list of stations for the Genre yyy.
 *
 * If you select a particular internet radio station, the receiver sends the following
 * request: /~prullman/irds.jsp?Format=xxx&ID=yyy
 * - We return the station yyy.
 * 
 * Responses are in XML.
 * 
 * @author Olivier Garrouste
 * Creation date: 2006-02-25
 */
public class NetTuneStreamHandler {

	private static final Logger logger = Logger.getLogger(NetTuneStreamHandler.class);

	public static void process(ProxyRequestValueObject proxyRequest,
			InputStream clientIn, OutputStream clientOut) {

		URL serverUrl;
		QueryString queryString;
		String strQueryString, response;

		// Get the the destination server URL.
		serverUrl = proxyRequest.getServerUrl();

		// Get the query string (the part after the '?' of the URL).
		strQueryString = serverUrl.getQuery();

		// Build the QueryString object.
		queryString = new QueryString(strQueryString);

		//
		// Generate a response.
		//
		String strId, strGenre, strFormat;

		strId = queryString.getParameter("ID");
		strGenre = queryString.getParameter("Genre");
		strFormat = queryString.getParameter("Format");
		logger.debug("ID=" + strId + ", Genre=" + strGenre + ", Format=" + strFormat + ".");

		// Refresh configuration file.
		ParseConfiguration.parse(Constant.CONFIGURATION_FILE);
		
		// Request a particular radio station.
		if ( strId != null ) {
			response = NetTuneProtocol.getRadio(strId);
		}
		// Request a genre.
		else if ( strGenre != null ) {
			response = NetTuneProtocol.getGenre(strGenre);
		}
		// Request main menu.
		else {
			// Build the response result.
			response = NetTuneProtocol.getMainMenu();
		}

		logger.debug("Send to client=" + response + ".");

		// Read response from server and forward to client.
		Net.write(response.getBytes(), clientOut);
		logger.debug("End sending to client.");
	}

}
