package util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;

/**
 * Networking utility class.
 * 
 * @author Olivier Garrouste.
 * Creation date: 2006-02-22.
 */
public class Net {

	private static final Logger logger = Logger.getLogger(Net.class);

	// End of line marker for HTTP protocol (CRLF).
	public static final String HTTP_END_OF_LINE_MARKER = "\r\n";


	/**
	 * Transfert from inputStream to outputStream as long as there is data to transfert.
	 * 
	 * @param inputStream The source stream to read from.
	 * @param outputStream The destination stream to write to.
	 */
	public static void transfert(InputStream inputStream, OutputStream outputStream) {
		byte buffer[];
		int nb;

		buffer = new byte[Constant.TRANSFERT_BUFFER_SIZE];

		try {
			while (true) {
				logger.debug("Before read().");
				// Read some bytes from input stream.
				nb = inputStream.read(buffer);
				logger.debug("After read(), nb=" + nb + ".");
	
				// End of stream ?
				if ( nb == -1 )
					break;

				// Write the bytes to the destination.
				outputStream.write(buffer, 0, nb);
			}
			outputStream.flush();
		}
		catch (Exception e) {
			// Socket exceptions are 'almost' normal operations, I log them in info mode.
			logger.info(e.toString());
		}
	}

	/**
	 * Transfert from inputStream to outputStream exactly size bytes.
	 * 
	 * @param inputStream The source stream to read from.
	 * @param outputStream The destination stream to write to.
	 * @param size Number of bytes to transfert.
	 * @return true if the stream continues, false if the end of stream has been reached
	 * or if an error occured.
	 */
	public static boolean transfert(InputStream inputStream, OutputStream outputStream,
			int size) {
		byte buffer[];
		int nb, sizeToRead;
		boolean ok;

		buffer = new byte[Constant.TRANSFERT_BUFFER_SIZE];

		try {
			while (true) {
				logger.debug("Avant read(), size=" + size + ".");

				// Limit the amount of bytes to read.
				if ( size > Constant.TRANSFERT_BUFFER_SIZE )
					sizeToRead = Constant.TRANSFERT_BUFFER_SIZE;
				else
					sizeToRead = size;

				// Read some bytes from input stream.
				nb = inputStream.read(buffer, 0, sizeToRead);
				logger.debug("Aprs read(), nb=" + nb + ".");
	
				// End of stream ?
				if ( nb == -1 ) {
					ok = false;
					break;
				}

				// Write the bytes to the destination.
				outputStream.write(buffer, 0, nb);
				size -= nb;

				// If we read enough data, exit.
				if ( size == 0 ) {
					ok = true;
					break;
				}
			}
			outputStream.flush();
		}
		catch (Exception e) {
			logger.error("ERROR : " + e.toString(), e);
			ok = false;
		}

		return ok;
	}

	/**
	 * Read a frame from the inputStream. The method returns if the data received exceed
	 * minSize or if there the end of stream has been reached.
	 * 
	 * @param inputStream The stream to read data from.
	 * @param minSize
	 * @return The data read from the stream. The returned buffer may be smaller than
	 * minSize if the end of stream has been reached. If we are not at the end of the
	 * stream the returned buffer contains at least minSize bytes but never less. 
	 */
	/*public static byte[] readBytes(InputStream inputStream, int minSize) {
		ByteArrayOutputStream byteArrayOutputStream;
		byte buffer[];
		int nb, totalReadBytes;
		
		totalReadBytes = 0;
		buffer = new byte[Constant.TRANSFERT_BUFFER_SIZE];

		// Create a dynamic byte buffer.
		byteArrayOutputStream = new ByteArrayOutputStream();

		try {
			while (true) {
				// Read some bytes from input stream.
				logger.debug("Before read().");
				nb = inputStream.read(buffer);
				logger.debug("After read(), nb=" + nb + ".");
	
				// End of stream ?
				if ( nb == -1 )
					break;

				// Write the input stream into the buffer.
				byteArrayOutputStream.write(buffer, 0, nb);
				totalReadBytes += nb;

				// If we read enough data, analyse it.
				if ( totalReadBytes >= minSize )
					break;
			}
		}
		catch (Exception e) {
			logger.error("ERROR : " + e.toString(), e);
		}

		return byteArrayOutputStream.toByteArray();
	}*/

	/**
	 * Read a frame from the inputStream. 
	 * 
	 * @param inputStream The stream to read data from.
	 * @return The data read from the stream or null if an error occured. If the end of
	 * the stream has been reached, return an empty buffer. 
	 */
	public static byte[] read(InputStream inputStream) {
		byte buffer[], response[];
		int nb;
		
		buffer = new byte[Constant.TRANSFERT_BUFFER_SIZE];

		try {
			// Read some bytes from input stream.
			logger.debug("Before read().");
			nb = inputStream.read(buffer);
			logger.debug("After read(), nb=" + nb + ".");
	
			// End of stream ?
			if ( nb == -1 ) {
				return new byte[0];
			}

			response = new byte[nb];
			System.arraycopy(buffer, 0, response, 0, nb);
		}
		catch (Exception e) {
			logger.error("ERROR : " + e.toString(), e);
			response = null;
		}

		return response;
	}

	/**
	 * Read exactly size bytes from stream.
	 * 
	 * @param inputStream
	 * @param size The sier to read/
	 * @return The data read from the stream, the buffer might be smaller than size
	 * if the end of stream has been reached.
	 */
	public static byte[] read(InputStream inputStream, int size) {
		ByteArrayOutputStream byteArrayOutputStream;
		byte buffer[];
		int nb;
		
		buffer = new byte[Constant.TRANSFERT_BUFFER_SIZE];

		// Create a dynamic byte buffer.
		byteArrayOutputStream = new ByteArrayOutputStream();

		try {
			while (true) {
				// Read some bytes from input stream.
				logger.debug("Before read(), size=" + size + ".");
				nb = inputStream.read(buffer, 0, size);
				logger.debug("After read(), nb=" + nb + ".");
	
				// End of stream ?
				if ( nb == -1 )
					break;

				// Write the input stream into the buffer.
				byteArrayOutputStream.write(buffer, 0, nb);
				size -= nb;

				// If we read enough data, exit.
				if ( size == 0 )
					break;
			}
		}
		catch (Exception e) {
			logger.error("ERROR : " + e.toString(), e);
		}

		return byteArrayOutputStream.toByteArray();
	}

	/**
	 * Write the buffer in the output stream.
	 * 
	 * @param buffer The buffer to write.
	 * @param outputStream
	 */
	public static void write(byte buffer[], OutputStream outputStream) {

		try {
			outputStream.write(buffer);
			outputStream.flush();
		}
		catch (Exception e) {
			logger.error(e.toString(), e);
		}
	}

	/**
	 * Retourne le contenu du fichier pass en paramtre.
	 * @param filename Fichier  charger.
	 * @return Le contenu du fichier ou null s'il a y eu une erreur ou si
	 *   le fichier n'existe pas.
	 */
	public static byte[] readFileContent(String filename) {
		File file = null;
		int filesize;
		byte[] buffer;
		FileInputStream inputStream = null;
		boolean fileFound;

		try {
			// On teste si le fichier existe dans le cache disque.
			file = new File(filename);
			fileFound = file.isFile();

			if ( ! fileFound )
				return null;

			// On rcupre la taille du ficher.
			filesize = (int) file.length();

			inputStream = new FileInputStream(file);

			// Allocation du buffer qui va recevoir le contenu du fichier.
			buffer = new byte[filesize];

			// Lit le contenu du fichier.
			inputStream.read(buffer);

			inputStream.close();
		}
		catch (FileNotFoundException e) {
			logger.warn("Fichier non trouv (" + file.getName() + ").");
			
			if ( inputStream != null )
				try {
					inputStream.close();
				}
				catch (Exception e1) {}

			buffer =  null;
		}
		catch (Exception e) {
			logger.error("Exception : " + e.getMessage());

			if ( inputStream != null )
				try {
					inputStream.close();
				}
				catch (Exception e1) {}

			if ( file != null )
				logger.warn("Erreur sur le fichier " + file.getName());

			buffer = null;
		}

		return buffer;
	}

	/**
	 * Send a message to the outputStream using HTTP protocol.
	 * 
	 * @param outputStream The stream where to send the message.
	 * @param message Message to send.
	 */
	public static void sendHttp(OutputStream outputStream, String message) {
		StringBuilder str;
		
		str = new StringBuilder();

		// First the header.
		str.append("HTTP/1.0 200" + HTTP_END_OF_LINE_MARKER +
				  "Content-Type: text/plain" + HTTP_END_OF_LINE_MARKER +
				  HTTP_END_OF_LINE_MARKER);

		// Then the content.
		str.append(message);

		try {
			// Send the whole frame in one shot.
			outputStream.write(str.toString().getBytes());
			outputStream.flush();
		}
		catch (Exception e) {
			logger.error(e.toString(), e);
		}
	}

}
