package util;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import util.Constant.MIME_TYPE;
import valueObject.AbstractGenreValueObject;
import valueObject.GenreValueObject;
import valueObject.RadioValueObject;
import valueObject.SubgenreValueObject;
import dao.GenreDao;
import dao.RadioDao;
import dao.SubgenreDao;

/**
 * Build the request result for the Net-tune client. The XML is hard coded which is not
 * very clear. TODO : use org.w3c.dom.Document and XML stuff.
 * 
 * @author Olivier Garrouste.
 * Creation date: 2006-02-25.
 */
public class NetTuneProtocol {

	private static final Logger logger = Logger.getLogger(NetTuneProtocol.class);

	/**
	 * Encoding scheme for the generated XML documents.
	 */
	private static final String XML_ENCODING = "UTF-8";

	private static final String HTTP_HEADER =
		"HTTP/1.1 200 OK" + Net.HTTP_END_OF_LINE_MARKER +
		"Connection: close" + Net.HTTP_END_OF_LINE_MARKER +
		"Content-Type: text/xml" + Net.HTTP_END_OF_LINE_MARKER +
		"Cache-Control: no-cache" + Net.HTTP_END_OF_LINE_MARKER +
		Net.HTTP_END_OF_LINE_MARKER +
		"<?xml version=\"1.0\" encoding=\"" + XML_ENCODING + "\"?>\n";
	
	private static final String XML_RADIO_LIST_HEADER =
		"<IRStationList xmlns=\"http://www.xiva-net.com/schemas/InternetRadioDataServiceTest\" " +
		"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
		"xsi:schemaLocation=\"http://www.xiva-net.com/schemas/InternetRadioDataServiceTest\n" +
		"P:\\Xiva\\Documents\\Projects\\XIVA-N~1.0\\InternetRadio\\IRadioTest.xsd\">\n";

	
	/**
	 * Return the XML for the main menu of Net-Tune.
	 * 
	 * @return A XML string.
	 */
	public static String getMainMenu() {
		GenreValueObject genre;
		StringBuilder str;
		AbstractGenreValueObject genreList[];
		int i, nb;

		str = new StringBuilder();

		str.append(HTTP_HEADER);
		str.append(XML_RADIO_LIST_HEADER);
		str.append("\t<IROptions>\n"+
				   "\t\t<Genres>\n");

		genreList = GenreDao.getListOrderByName();

		nb = genreList.length;

		for ( i=0 ; i<nb ; i++ ) {
			genre = (GenreValueObject) genreList[i];

			str.append("\t\t\t<Genre code=\"");
			str.append(genre.getCode());
			str.append("\" type=\"genre\">\n" +
					   "\t\t\t\t<Name language=\"en\">");
			str.append(genre.getName());
			str.append("</Name>\n" +
					   "\t\t\t</Genre>\n");

			str.append(getMainMenuSubgenre(genre.getCode(), genre.getSubgenreList()));
		}

		str.append("\t\t</Genres>\n" +
				   "\t\t<Locations>\n" +
				   "\t\t\t<Location country=\"CS\" region=\"8053\"/>\n" +
				   "\t\t\t<Location country=\"AI\" region=\"6111\"/>\n" +
				   "\t\t\t<Location country=\"AR\" region=\"6414\"/>\n" +
				   "\t\t\t<Location country=\"AW\" region=\"6114\"/>\n" +
				   "\t\t\t<Location country=\"US\" region=\"6357\"/>\n" +
				   "\t\t</Locations>\n" +
				   "\t\t<Languages>\n" +
				   "\t\t\t<Language>en</Language>\n" +
				   "\t\t\t<Language>it</Language>\n" +
				   "\t\t\t<Language>ja</Language>\n" +
				   "\t\t</Languages>\n" +
				   "\t\t<Formats>\n" +
				   "\t\t\t<Format MIME=\"application/vnd.ms-asf\"/>\n" +
				   "\t\t\t<Format MIME=\"audio/mpeg\"/>\n" +
				   "\t\t\t<Format MIME=\"audio/x-mpegurl\"/>\n" +
				   "\t\t\t<Format MIME=\"audio/x-scpls\"/>\n" +
				   "\t\t</Formats>\n" +
				   "\t</IROptions>\n" +
				   "</IRStationList>");

		return str.toString();
	}

	/**
	 * Return the XML list for the given subgenres.
	 * 
	 * @param subgenreList An ArrayList of GenreValueObject.
	 * @return A XML string.
	 */
	private static String getMainMenuSubgenre(String strGenreCode, ArrayList subgenreList) {
		SubgenreValueObject subgenre;
		StringBuilder str;
		int i, nb;
		
		str = new StringBuilder();

		nb = subgenreList.size();
		
		for ( i=0 ; i<nb ; i++ ) {
			subgenre = (SubgenreValueObject) subgenreList.get(i);

			str.append("\t\t\t<Genre code=\"");
			str.append(subgenre.getCode());
			str.append("\" type=\"subgenre\">\n" +
					   "\t\t\t\t<Name language=\"en\">");
			str.append(subgenre.getName());
			str.append("</Name>\n" +
			   "\t\t\t</Genre>\n");
		}

		return str.toString();
	}

	/**
	 * Return the list of radio for for the given genre code (in XML).
	 * 
	 * @param strGenreCode
	 * @return A XML string.
	 */
	public static String getGenre(String strGenreCode) {
		GenreValueObject genre;
		SubgenreValueObject subgenre;
		RadioValueObject radio;
		//ArrayList subgenreList;
		RadioValueObject radioList[];
		StringBuilder str;
		String strId, strUrl;
		int i, nb;

		str = new StringBuilder();
		
		try {
			str.append(HTTP_HEADER);
			str.append(XML_RADIO_LIST_HEADER);
	
			// Retrieve the requested genre.
			genre = (GenreValueObject) GenreDao.getByCode(strGenreCode);
	
			// If the genre does not exists, check into subgenre list.
			if ( genre == null ) {
				subgenre = SubgenreDao.getByCode(strGenreCode);
	
				if ( subgenre == null ) {
					logger.warn("Genre code \"" + strGenreCode + "\" not found.");
					str.append("</IRStationList>");
					return str.toString();
				}
				str.append(getSubgenre(subgenre));
				str.append("</IRStationList>");
				return str.toString();
			}
	
			// Get the list of radio from the main genre (it will be displayed under the
			// Miscellaneous subgenre on the receiver).
			radioList = genre.getRadioList();
	
			nb = radioList.length;
	
			for ( i=0 ; i<nb ; i++ ) {
				radio = (RadioValueObject) radioList[i];
	
				// Convert the radio identifier into a four digit number.
				strId = Formater.intToStr4(radio.getId());
				
				// Encode URL correctly.
				strUrl = radio.getUrl();
				strUrl = Formater.urlToXml(strUrl);
	
				str.append(
					"\t<IRStation>\n" +
					"\t\t<ID>").append(strId).append("</ID>\n" +
					"\t\t<Name reliability=\"100\">").append(radio.getName()).append("</Name>\n" +
					"\t\t<Genre code=\"").append(strGenreCode).append("\"/>\n" +
					"\t\t<Location country=\"US\" location=\"\" region=\"6357\"/>\n" +
					"\t\t<Language></Language>\n" +
					"\t\t<URL MIME=\"").append(getXmlMimeType(radio.getMimeType()));
				str.append("\">").append(strUrl).append("</URL>\n" +
					"\t</IRStation>\n");
			}
	
			str.append("</IRStationList>");
		}
		catch (Exception e) {
			logger.error(e.toString(), e);
		}

		return str.toString();
	}

	/**
	 * Return the XML for the given subgenre.
	 * 
	 * @param subgenre A SubgenreValueObject.
	 * @return A XML string.
	 */
	public static String getSubgenre(AbstractGenreValueObject subgenre) {
		RadioValueObject radio;
		StringBuilder str;
		RadioValueObject radioList[];
		String strId, strUrl;
		int i, nb;

		str = new StringBuilder();

		// Get the radio list of this subgenre.
		radioList = subgenre.getRadioList();

		nb = radioList.length;

		for ( i=0 ; i<nb ; i++ ) {
			radio = (RadioValueObject) radioList[i];

			// Convert the radio identifier into a four digit number.
			strId = Formater.intToStr4(radio.getId());

			// Encode URL correctly.
			strUrl = radio.getUrl();
			strUrl = Formater.urlToXml(strUrl);

			str.append(
					"\t<IRStation>\n" +
					"\t\t<ID>").append(strId).append("</ID>\n" +
					"\t\t<Name reliability=\"100\">").append(radio.getName()).append("</Name>\n" +
					"\t\t<Genre code=\"").append(subgenre.getCode()).append("\"/>\n" +
					"\t\t<Location country=\"US\" location=\"\" region=\"6357\"/>\n" +
					"\t\t<Language></Language>\n" +
					"\t\t<URL MIME=\"").append(getXmlMimeType(radio.getMimeType()));
			str.append("\">").append(strUrl).append("</URL>\n" +
					"\t</IRStation>\n");
		}

		return str.toString();
	}

	/**
	 * Return the radio of the given id.
	 * 
	 * @param strId Radio identifier.
	 * @return A XML string.
	 */
	public static String getRadio(String strId) {
		RadioValueObject radio;
		StringBuilder str;
		String strUrl;
		Integer idRadio;

		str = new StringBuilder();

		try {
			// Convert the radio id to an Integer.
			idRadio = new Integer(strId);

			radio = RadioDao.getById(idRadio);

			if ( radio == null ) {
				logger.warn("The radio with id=" + idRadio + " not found.");
				return "";
			}

			// Encode URL correctly.
			strUrl = radio.getUrl();
			strUrl = Formater.urlToXml(strUrl);

			str.append(HTTP_HEADER);
			str.append(XML_RADIO_LIST_HEADER);
			str.append(
					"\t<IRStation>\n" +
					"\t\t<ID>").append(strId).append("</ID>\n" +
					"\t\t<Name reliability=\"100\">").append(radio.getName()).append("</Name>\n" +
					"\t\t<Genre code=\"").append(radio.getGenre().getCode()).append("\"/>\n" +
					"\t\t<Location country=\"US\" location=\"\" region=\"6357\"/>\n" +
					"\t\t<Language></Language>\n" +
					"\t\t<URL MIME=\"").append(getXmlMimeType(radio.getMimeType()));
			str.append("\">").append(strUrl).append("</URL>\n" +
					"\t</IRStation>\n" +
					"</IRStationList>\n");
		}
		catch (Exception e) {
			logger.warn(e.toString() + ", invalid radio number " + strId + ".");
		}

		return str.toString();
	}
	
	/**
	 * Return the XML string corresponding to the given mime type id.
	 * 
	 * @param mimeType The MIME type identifier.
	 * @return The MIME type string.
	 */
	private static String getXmlMimeType(MIME_TYPE mimeType) {
		String str;

		switch ( mimeType ) {
		case MS_ASF:
			str = "application/vnd.ms-asf";
			break;

		case MPEG:
			str = "audio/mpeg";
			break;

		case PLS:
			str = "audio/x-scpls";
			break;

		case M3U:
			str = "audio/x-mpegurl";
			break;

		default:
			str = "audio/mpeg";
		}

		return str;
	}
}
