package util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import dao.GenreDao;
import dao.RadioDao;
import dao.SubgenreDao;

import util.Constant.MIME_TYPE;
import valueObject.GenreValueObject;
import valueObject.RadioConfigurationValueObject;
import valueObject.RadioValueObject;
import valueObject.SubgenreValueObject;

/**
 * This classe reads the radio configuration file.
 * 
 * @author Olivier Garrouste
 * Creation date: 2006-02-25.
 */
public class ParseConfiguration {

	private static final Logger logger = Logger.getLogger(ParseConfiguration.class);

	private static final String DELIMITER = ";";

	// The last modification date of the file.
	private static long lastModificationDate = -1;



	/**
	 * Parse the file fileName. This file contains a list of radio stations (see the
	 * manual for more information).
	 * 
	 * @param fileName The file to parse.
	 * @return An ArrayList of RadioConfigurationValueObject. Return an empty list if an error
	 * occured.
	 */
	public static void parse(String fileName) {
		File file;
		FileReader fileReader;
		BufferedReader bufferedReader;
		RadioConfigurationValueObject radio;
		ArrayList<RadioConfigurationValueObject> radioList;
		String line, content, token[];
		MIME_TYPE mimeType;
		int lineNumber, numberOfFields;

		logger.debug("Begin parse().");
		file = new File(fileName);

		// Check if the file has been modified since last parsing.
		if ( file.lastModified() <= lastModificationDate ) {
			logger.debug("End parse() (file not modified).");
			return;
		}

		// The file has been modified (or it is the first access). We read it and store
		// it in radioList.
		logger.debug("File modified, parsing " + fileName + ".");

		lineNumber = 0;

		try {
			// Create the file readers.
			fileReader = new FileReader(fileName);
			bufferedReader = new BufferedReader(fileReader);

			// Create the temporary list to be populated (forbid access to clients
			// trying to connect while reading the file).
			radioList = new ArrayList<RadioConfigurationValueObject>();

			while (true) {
				line = bufferedReader.readLine();
				lineNumber++;

				// End of file.
				if ( line == null )
					break;

				// Empty line. Continue reading.
				if ( line.length() == 0 )
					continue;

				// Create a new RadioValueObject.
				radio = new RadioConfigurationValueObject();

				// line contains "genre;subgenre;name;url", split it arround semicolon.
				token = line.split(DELIMITER);

				// Get the genre.
				content = token[0];
				content = content.trim();

				// If it is a comment line, skip.
				if ( content.length() > 0 )
					if ( content.charAt(0) == '#' )
						continue;

				numberOfFields = token.length;
				// This not a comment, we must have 4 or 5 fields.
				if ( (numberOfFields < 4) || (numberOfFields > 5) ) {
					logger.error("File " + fileName + ", line " + lineNumber +
							" : 4 or 5 fields required, line skipped.");
					continue;
				}

				if ( content.length() == 0 ) {
					logger.warn("File " + fileName + ", line " + lineNumber +
						" : genre not specified, using \"Genre" + lineNumber + "\".");
					radio.setGenre("Genre" + lineNumber);
				}
				else
					radio.setGenre(content);

				// Get the subgenre.
				content = token[1];
				content = content.trim();
				radio.setSubgenre(content);

				// Get the radio name.
				content = token[2];
				content = content.trim();
				if ( content.length() == 0 ) {
					logger.warn("File " + fileName + ", line " + lineNumber +
						" : name not specified, using \"Name" + lineNumber + "\".");
					radio.setName("Name" + lineNumber);
				}
				else
					radio.setName(content);

				// Get the url.
				content = token[3];
				content = content.trim();
				if ( content.length() == 0 ) {
					logger.error("File " + fileName + ", line " + lineNumber +
							" : URL required, line skipped.");
					continue;
				}
				else
					radio.setUrl(content);

				// Get the type of stream.
				if ( numberOfFields == 4 ) {
					// Try to determine the type of the stream from the URL.
					mimeType = autodetectMimeType(radio.getUrl());
				}
				else {
					// The fifth field is given.
					content = token[4];
					content = content.trim();
					if ( content.length() == 0 )
						// Empty field, try to determine the type of the stream from the
						// URL.
						mimeType = autodetectMimeType(radio.getUrl());
					else
						mimeType = getMimeType(content);
				}
				// Set the MIME type of the stream.
				radio.setMimeType(mimeType);

				// Add the radio to the list.
				radioList.add(radio);
			}

			// Update the last modification date.
			lastModificationDate = file.lastModified();

			// Update the database with the retrieved radio from the file.
			updateDatabase(radioList);
		}
		catch (Exception e) {
			logger.error("parse(), fileName=" + fileName + ", lineNumber=" + lineNumber + ".", e);
		}

		logger.debug("End parse().");
	}

	/**
	 * Recreate the database with the given radio list.
	 * Database organization :
	 * - The database is composed of radio (RadioValueObject).
	 * - A radio (RadioValueObject) owns one genre (GenreValueObject) or one subgenre
	 * (SubgenreValueObject).
	 * - A genre (GenreValueObject) might owns subgenre (SubgenreValueObject).
	 * 
	 * Radio, genres and subgenres are stored in xxxDao classes.
	 *  
	 * @param radioConfigurationList An ArrayList of RadioValueObject.
	 */
	private static synchronized void updateDatabase(ArrayList<RadioConfigurationValueObject> radioConfigurationList) {
		RadioConfigurationValueObject radioConfiguration;
		RadioValueObject radio;
		GenreValueObject genre;
		SubgenreValueObject subgenre;
		String strSubgenre;
		int i, nb;

		// Clear the database.
		GenreDao.clear();
		SubgenreDao.clear();
		RadioDao.clear();

		nb = radioConfigurationList.size();

		for ( i=0 ; i<nb ; i++ ) {
			radioConfiguration = (RadioConfigurationValueObject) radioConfigurationList.get(i);

			// Create the radio object.
			radio = new RadioValueObject();
			radio.setName(radioConfiguration.getName());
			radio.setUrl(radioConfiguration.getUrl());
			radio.setMimeType(radioConfiguration.getMimeType());

			// Get the genre from the database.
			genre = (GenreValueObject) GenreDao.getByName(radioConfiguration.getGenre());

			// If the genre does not exists, we create it.
			if ( genre == null ) {
				genre = new GenreValueObject();
				genre.setName(radioConfiguration.getGenre());

				// Insert this genre object into the global database.
				GenreDao.insert(genre);
			}

			strSubgenre = radioConfiguration.getSubgenre();

			// Create the subgenre if needed.
			if ( strSubgenre.length() != 0 ) {

				// Check if the subgenre already exists the current genre.
				subgenre = SubgenreDao.getByName(genre.getSubgenreList(), strSubgenre);

				// If it does not exist, create it.
				if ( subgenre == null ) {
					subgenre = new SubgenreValueObject(genre);
					subgenre.setName(strSubgenre);
	
					// Add it to the genre.
					genre.addSubgenre(subgenre);

					// Insert this subgenre object into the global database.
					SubgenreDao.insert(subgenre);
				}

				// Set the subgenre.
				radio.setGenre(subgenre);

				// Add the radio into that subgenre.
				subgenre.addRadio(radio);
			}
			else {
				// No subgenre specified.

				// Set the genre.
				radio.setGenre(genre);

				// Add the radio into that genre.
				genre.addRadio(radio);
			}

			// Insert this radio object into the global database.
			RadioDao.insert(radio);
		} // of for.
		GenreDao.dumpGenreByCode();
	}

	/**
	 * Return the mime type from the given URL. If it is not possible to determine URL,
	 * return MIME_TYPE.MPEG. If the URL is invalid, return MIME_TYPE.UNKNOWN.
	 * 
	 * @param strMimeType
	 * @return An value from MIME_TYPE.xxx.
	 */
	private static MIME_TYPE autodetectMimeType(String strUrl) {
		String strExtension;
		int nb, i;

		nb = strUrl.length();

		if ( nb < 4 )
			return MIME_TYPE.UNKNOWN;

		// Remove the query string (part after the question mark of the URL).
		i = strUrl.indexOf("?");
		if ( i != -1 ) {
			if ( i >= 4 )
				// Question mark found, keep the four characters just before.
				strExtension = strUrl.substring(i-4, i);
			else
				// Keep only the last four characters.
				strExtension = strUrl.substring(nb - 4);
		}
		else
			// Keep only the last four characters.
			strExtension = strUrl.substring(nb - 4);

		if ( ".asf".equals(strExtension) )
			return MIME_TYPE.MS_ASF;
		if ( ".asx".equals(strExtension) )
			return MIME_TYPE.MS_ASF;
		if ( ".m3u".equals(strExtension) )
			return MIME_TYPE.M3U;
		if ( ".mp3".equals(strExtension) )
			return MIME_TYPE.MPEG;
		if ( ".pls".equals(strExtension) )
			return MIME_TYPE.PLS;

		logger.info("Unknown stream type: \"" + strUrl + "\", defaulting to mpeg.");

		return MIME_TYPE.MPEG;
	}

	/**
	 * Return the mime type from the given string. If the given mime type is invalid,
	 * MIME_TYPE_UNKNOWN is returned.
	 * 
	 * @param strMimeType
	 * @return An int of MIME_TYPE_xxx.
	 */
	private static MIME_TYPE getMimeType(String strMimeType) {
		if ( strMimeType == null )
			return MIME_TYPE.UNKNOWN;
		else if ( "ms-asf".equals(strMimeType) )
			return MIME_TYPE.MS_ASF;
		else if ( "pls".equals(strMimeType) )
			return MIME_TYPE.PLS;
		else if ( "mpeg".equals(strMimeType) )
			return MIME_TYPE.MPEG;
		else if ( "m3u".equals(strMimeType) )
			return MIME_TYPE.M3U;
		else
			logger.warn("Unknown stream type \"" + strMimeType + "\".");

		return MIME_TYPE.UNKNOWN;
	}}
