package util;

import java.io.InputStream;

import org.apache.log4j.Logger;

import util.Constant.Protocol;
import valueObject.ResponseValueObject;
import valueObject.ProxyRequestValueObject;

/**
 * Parse the stream sent by the server to find out the protocol used.
 * 
 * @author Olivier Garrouste.
 * Creation date: 2006-03-26
 */
public class ProtocolAnalyzer {

	private static final Logger logger = Logger.getLogger(ProtocolAnalyzer.class);
	
	/**
	 * Analyze the protocol used by the server. Result is wrapped into the
	 * returned ResponseValueObject.
	 * 
	 * @param inputStream The source stream to read from.
	 * @return A ResponseValueObject.
	 */
	public static ResponseValueObject analyze(ProxyRequestValueObject proxyRequest) {
		InputStream inputStream;
		byte frame[];
		String response, headerArray[], headerName, headerValue, str;
		ResponseValueObject protocol;
		int statusCode, i, j, nb;

		logger.debug("Analyze protocol from server.");

		// Create the ResponseValueObject object.
		protocol = new ResponseValueObject();

		// Read a frame of data from the server response.
		inputStream = proxyRequest.getServerInputStream();
		frame = Net.read(inputStream);

		// If no data were read (server problem).
		if ( frame.length == 0 ) {
			logger.info("No data could be read from server.");

			protocol.setIdProtocol(Protocol.UNKNOWN);
			return protocol;
		}

		// Convert the bytes read into a string.
		response = new String(frame);

		logger.debug("Received headers=\"" + getHeaders(response) + "\".");

		// Store the read data in the protocol object.
		protocol.setData(frame);

		headerArray = response.split(Net.HTTP_END_OF_LINE_MARKER);
		nb = headerArray.length;

		str = headerArray[0].substring(0, 4);
		if ( ! "ICY ".equalsIgnoreCase(str) ) {
			// Unknown protocol, we can't do anything with it.
			logger.info("Unknown protocol, header[0]==\"" + headerArray[0] + "\".");

			protocol.setIdProtocol(Protocol.UNKNOWN);
			return protocol;
		}

		// ICY protocol !
		logger.info("ICY protocol, received header[0]=\"" + headerArray[0] + "\".");
		protocol.setIdProtocol(Protocol.ICY);

		// Parse the status code.
		str = headerArray[0].substring(4, 7);
		try {
			statusCode = Integer.parseInt(str);
		}
		catch (Exception e) {
			logger.warn("Invalid status code received from server \"" + str + "\".");
			statusCode = 0;
		}

		protocol.setStatusCode(statusCode);

		// Parse the header line. Skip the the first line containing the status code.
		for ( i=1 ; i<nb ; i++ ) {
			str = headerArray[i].trim();

			// Empty line means end of headers.
			if ( str.length() == 0 )
				break;

			// Find the separator. A header is string like "name: value".
			j = str.indexOf(":");

			// A line without semicolon. It is invalid, we don't do anything.
			if ( j == -1 ) {
				logger.warn("Don't known what to do with header \"" + str + "\", ignore.");
				continue;
			}

			headerName = str.substring(0, j).toLowerCase();
			headerValue = str.substring(j+1).trim();

			protocol.setHeader(headerName, headerValue);
		}

		return protocol;
	}

	/**
	 * Retrieve HTTP headers (if exist) from the raw data.
	 * 
	 * @param rawData The beginning of a TCP communication.
	 * @return The headers if found, else null;
	 */
	private static String getHeaders(String rawData) {
		int i;

		// The end of headers is "\r\n\r\n".
		i = rawData.indexOf(Net.HTTP_END_OF_LINE_MARKER + Net.HTTP_END_OF_LINE_MARKER);

		// End of headers not found.
		if ( i == -1 )
			return null;

		return rawData.substring(0, i+2);
	}
}
