package util;

import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * Handles query strings.
 * 
 * @author Olivivier Garrouste
 * Creation date: 2006-02-25
 */
public class QueryString {

	HashMap<String,String[]> hashmapParameters;
	
	public QueryString(String strQueryString) {
		if ( strQueryString != null )
			hashmapParameters = parseQueryString(strQueryString);
		else
			hashmapParameters = new HashMap<String,String[]>();
	}

	/**
	 * Returns the value of a request parameter as a String, or null if the parameter
	 * does not exist. You should only use this method when you are sure the parameter
	 * has only one value. If the parameter might have more than one value, use
	 * getParameterValues().
	 * 
	 * @param name a String specifying the name of the parameter.
	 * @return a String representing the single value of the parameter.
	 */
	public String getParameter(String name) {
		String strArray[];
		
		strArray = hashmapParameters.get(name);

		if ( strArray == null )
			return null;

		return strArray[0];
	}

	/**
	 * Returns an array of String objects containing all of the values the given request
	 * parameter has, or null if the parameter does not exist.
	 *  
	 * @param name a String containing the name of the parameter whose value is
	 * requested.
	 * @return an array of String objects containing the parameter's values.
	 */
	public String[] getParameterValues(String name) {
		return hashmapParameters.get(name);
	}

	/**
	 * Source code of deprecated javax.servlet.http.parseQueryString() method.
	 * Modified by Olivier Garrouste.
	 * 
	 * @param s Query string.
	 * @return A HashMap<String,String[]> of key-value pairs.
	 * @throws IllegalArgumentException if the QueryString is null;
	 */
	private static HashMap<String,String[]> parseQueryString(String s) {
		StringTokenizer stringTokenizer;
		HashMap<String,String[]> hashMap;
		String valArray[] = null;
		String oldValues[];
		String key, value, chunk;
		int nb, i, pos;

		if ( s == null )
			throw new IllegalArgumentException();

		hashMap = new HashMap<String,String[]>();
		

		stringTokenizer = new StringTokenizer(s, "&");
		while ( stringTokenizer.hasMoreTokens() ) {
			// Get next chunk "attribute=value".
			chunk = stringTokenizer.nextToken();

			pos = chunk.indexOf('=');
			if (pos == -1)
				throw new IllegalArgumentException();

			key = parseName(chunk.substring(0, pos));
			value = parseName(chunk.substring(pos + 1, chunk.length()));

			// The hashmap already contains a key/value pair.
			if ( hashMap.containsKey(key) ) {
				oldValues = hashMap.get(key);
				
				nb = oldValues.length;
				valArray = new String[nb + 1];
				for (i=0 ; i<nb ; i++)
					valArray[i] = oldValues[i];

				valArray[nb] = value;
			}
			else {
				valArray = new String[1];
				valArray[0] = value;
			}
			hashMap.put(key, valArray);
		}

		return hashMap;
	}

	private static String parseName(String s) {
		StringBuilder str;
		String rest;
		int i, nb;
		char c;

		str = new StringBuilder();
		nb = s.length();
		
		for ( i = 0; i < nb ; i++ ) {
			c = s.charAt(i);

			switch (c) {
			case 43: // '+'
				str.append(' ');
				break;

			case 37: // '%'
				try {
					str.append((char) Integer.parseInt(s.substring(i + 1, i + 3), 16));
					i += 2;
					break;
				}
				catch (NumberFormatException e) {
					throw new IllegalArgumentException();
				}
				catch (StringIndexOutOfBoundsException e) {
					rest = s.substring(i);
					str.append(rest);
					if (rest.length() == 2)
						i++;
				}
				break;

			default:
				str.append(c);
				break;
			}
		}

		return str.toString();
	}
}
