package util;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;

import org.apache.log4j.Logger;

import valueObject.ProxyRequestValueObject;

/**
 * Parse proxy style HTTP-request.
 * 
 * @author Olivier Garrouste.
 * Creation date: 2006-02-22.
 */
public class RequestParser {

	private static final Logger logger = Logger.getLogger(RequestParser.class);

	/**
	 * Read the client request from the input stream and fills the
	 * ProxyRequestValueObject which is returned.
	 * 
	 * @param ProxyRequestValueObject 
	 * @return A ProxyRequestValueObject. Initialize the fields clientInputStream,
	 * clientOutputStream, request and serverUrl. Return null if an error occured.
	 */
	public static ProxyRequestValueObject build(InputStream clientIn, OutputStream clientOut) {
		ProxyRequestValueObject proxyRequest;
		URL serverURL;
		BufferedReader clientInputStream;
		String line, normalizedLine, strServerUrl, strHttpVersion;
		int i;

		// Instanciate a ProxyRequestValueObject.
		proxyRequest = new ProxyRequestValueObject();
		proxyRequest.setClientInputStream(clientIn);
		proxyRequest.setClientOutputStream(clientOut);

		try {
			// Get client input.
			clientInputStream = new BufferedReader(new InputStreamReader(clientIn));

			// Read the response result (first line).
			line = clientInputStream.readLine();

			// End of client request.
			if ( line == null )
				return proxyRequest;

			// Remove heading and tailing white spaces.
			normalizedLine = line.trim();

			// Is it a GET request ?
			if ( normalizedLine.toUpperCase().startsWith("GET") ) {
				// We found a proxy get request that should look something like this:
				// "GET http://server.com:port/path/file HTTP 1.0"
				// We must remove the tailing "HTTP 1.x" part.

				// Remove the starting "GET".
				strServerUrl = normalizedLine.substring(3);

				// Find the last occurence of "HTTP" in the url. 
				i = strServerUrl.toUpperCase().lastIndexOf("HTTP");

				// If the "HTTP" string has been found.
				if ( i >= 0 ) {
					strHttpVersion = " " + strServerUrl.substring(i).trim();
					strServerUrl = strServerUrl.substring(0, i).trim();
				}
				else
					strHttpVersion = "";

				// Create a java URL object from the server url obtained.
				serverURL = new URL(strServerUrl);

				// Set the URL of the destination server.
				proxyRequest.setServerUrl(serverURL);

				// Transform the proxy GET request into a standard HTTP request.
				proxyRequest.appendRequestLine("GET " + serverURL.getFile() + strHttpVersion);

				logger.debug("GET " + strServerUrl + strHttpVersion);
			}
			else
				// It's not a GET request.
				// TODO : handle other requests.
				return null;

			// Read the headers.
			while (true) {
				// Read a line from the client input.
				line = clientInputStream.readLine();
	
				// End of client request.
				if ( line == null )
					break;
	
				// Empty line: end of request.
				if ( line.length() == 0 )
					break;

				proxyRequest.appendRequestLine(line);
			}
		}
		catch (Exception e) {
			logger.error(e.toString(), e);
			proxyRequest = null;
		}

		return proxyRequest;
	}

}
