package valueObject;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import util.Net;

/**
 * A request object handled by the proxy.
 * This object contains :
 * - the URL requested by the client.
 * - the request made by the client (the part after the '?' of the URL).
 * - the input and output stream of the client that made a request.
 * - the input and output stream of the destination server.
 * 
 * @author Olivier Garrouste
 * Creation: 2006-02-26
 */
public class ProxyRequestValueObject {

	/**
	 * The request to be forwarded to the server. 
	 */
	private StringBuilder request;

	/**
	 * The URL of the destination server.
	 */
	private URL serverUrl;

	/**
	 * Data stream from client to proxy.
	 */
	private InputStream clientInputStream;

	/**
	 * Data stream from proxy to client.
	 */
	private OutputStream clientOutputStream;

	/**
	 * Data stream from server to proxy.
	 */
	private InputStream serverInputStream;

	/**
	 * Data stream from proxy to server.
	 */
	private OutputStream serverOutputStream;

	/**
	 * Constructor.
	 */
	public ProxyRequestValueObject() {
		request = new StringBuilder();
	}

	/**
	 * Returns the server URL only for a GET method. For methods different of GET the
	 * returned value is null.
	 * 
	 * @return Returns the serverUrl.
	 */
	public URL getServerUrl() {
		return serverUrl;
	}

	/**
	 * @param serverUrl The serverUrl to set.
	 */
	public void setServerUrl(URL serverUrl) {
		this.serverUrl = serverUrl;
	}

	/**
	 * Build the HTTP request that will be used to contact the server. This method
	 * appends a new line to the request private field. Use this method to add the
	 * request-line ("GET / HTTP 1.1") then use it again to add header fields
	 * ("Host: www.server.com").
	 * 
	 * @param requestLine A line to add to the request, the method appends a "\r\n"
	 * at the end.
	 */
	public void appendRequestLine(String requestLine) {
		request.append(requestLine);
		request.append(Net.HTTP_END_OF_LINE_MARKER);
	}

	/**
	 * @return Returns the request.
	 */
	public String getRequest() {
		return request.toString();
	}

	/**
	 * @return Returns the serverInputStream.
	 */
	public InputStream getServerInputStream() {
		return serverInputStream;
	}

	/**
	 * @param serverInputStream The serverInputStream to set.
	 */
	public void setServerInputStream(InputStream serverInputStream) {
		this.serverInputStream = serverInputStream;
	}

	/**
	 * @return Returns the serverOutpuStream.
	 */
	public OutputStream getServerOutputStream() {
		return serverOutputStream;
	}

	/**
	 * @param serverOutpuStream The serverOutpuStream to set.
	 */
	public void setServerOutputStream(OutputStream serverOutpuStream) {
		this.serverOutputStream = serverOutpuStream;
	}

	/**
	 * @return Returns the clientInputStream.
	 */
	public InputStream getClientInputStream() {
		return clientInputStream;
	}

	/**
	 * @param clientInputStream The clientInputStream to set.
	 */
	public void setClientInputStream(InputStream clientInputStream) {
		this.clientInputStream = clientInputStream;
	}

	/**
	 * @return Returns the clientOutputStream.
	 */
	public OutputStream getClientOutputStream() {
		return clientOutputStream;
	}

	/**
	 * @param clientOutputStream The clientOutputStream to set.
	 */
	public void setClientOutputStream(OutputStream clientOutputStream) {
		this.clientOutputStream = clientOutputStream;
	}

}
